//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [tle] = CL_tle_load(fnames)
// Loads TLEs from files
//
// Calling Sequence
// tle = CL_tle_load(fnames)
//
// Description
// <itemizedlist>
// <listitem>
// <p>Loads two-line elements from files. </p>
// <p>The file names are considered in sequence. </p>
// <p>The files should contain two-line elements on 2 or 3 lines: 
// two data lines (lines #1 and #2) and one optional description line above line #1. </p>
// <p></p>
// <p>The status field is set to a non-zero value if parsing errors occur (see <link linkend="CL_tle_parse">CL_tle_parse</link>). </p>
// <p></p> 
// </listitem>
// <listitem>
// <p><b>Notes:</b></p>
// <p>The spaces at the beginning or end of a line are ignored. </p> 
// <p>For the files to be read correctly, one of the following conditions should be true: </p>
// <p>-No description line is present in the file </p>
// <p>-The description line does not begin with "1" or "2" or its length is not 69 </p>
// <p></p> 
// </listitem>
// </itemizedlist>
//
// Parameters
// fnames: (string) File names or paths (1xP)
// tle: Structure containing all the TLEs present in the files (size N)
//
// Authors
// CNES - DCT/SB/MS
//
// See also
// CL_tle_parse
// CL_tle_new
//
// Examples
// fnames = CL_path("tle_examples.txt", CL_home(), "all"); 
// tle = CL_tle_load(fnames); 
// // show 2nd element of TLE structure
// tle(2)
// // show all eccentricities 
// tle.ecc
// // select TLE indices such that eccentricity is more than 0.01
// I = find(tle.ecc > 0.01); 
// tle2 = tle(I)



// internal function: loads TLEs from ONE file 
// fname: string (1x1)
function [tle] = tle_load(fname)
  LEN = 69; 

  if (~isfile(fname))
    CL__error("File not found: " + fname); 
  end

  M = stripblanks(mgetl(fname), %t);

  I1 = find(part(M,1) == "1" & length(M) == LEN); 
  I2 = find(part(M,1) == "2" & length(M) == LEN); 

  if (length(I1) <> length(I2))
    CL__error("Invalid TLE file"); 
  end

  if (I1 == [])
    tle = CL__tle_create(); 
    return; 
  end

  if (find(I2-I1 <> 1) <> [])
    CL__error("Invalid TLE file"); 
  end

  str = [M(I1), M(I2)]'; 

  // Line with description: above TLE line 1
  // except if it is a data line (begins with 1 or 2 and composed of 69 characters)
  Idesc = max(I1-1,1); 
  desc = M(Idesc)'; 
  I = find((part(desc,1) == "1" | part(desc,1) == "2") & length(desc) == LEN); 
  if (I <> [])
    desc(I) = ""; 
  end

  tle = CL_tle_parse(str, desc); 

endfunction


// empty by default
tle = CL__tle_create(); 

if (fnames == [])
  return;  
end

if (typeof(fnames) <> "string")
  CL__error("Wrong type for input argument ''fnames''"); 
end

for k = 1 : size(fnames,2)
  fname = fnames(k); 

  // creates TLE
  tle1 = tle_load(fname); 

  // add to previously read TLEs
  tle = [tle, tle1]; 

end

endfunction
